<?php

namespace TDevAgency\Erp\Controllers;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\View;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Erp\Models\Customer;
use TDevAgency\Erp\Models\CustomerAddress;
use Throwable;

class CustomerController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function index()
    {
        $customers = Customer::sortable()
            ->container()
            ->search()
            ->paginate(20);
        return View::make('erp::pages.customers.index')->with(compact('customers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $view = View::make('erp::pages.customers.single');
        $formMethod = 'POST';
        $formRoute = route('cms.customers.store');
        $customer = new Customer();
        $view->with(compact('formRoute', 'formMethod', 'customer'));
        return $view;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return RedirectResponse
     * @throws Throwable
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'email' => 'nullable|email',
            'phone' => 'required',
            'first_name' => 'required|string',
            'last_name' => 'required|string',
        ]);

        DB::transaction(function () use ($request) {
            $customer = new Customer();

            $customer->email = $request->get('email') ?? null;
            $customer->phone = $request->get('phone') ?? null;
            $customer->last_name = $request->get('last_name') ?? null;
            $customer->first_name = $request->get('first_name') ?? null;
            $customer->customer_comment = $request->get('customer_comment') ?? null;
            $customer->save();

            if ($request->has('customer_addresses') && is_array($request->get('customer_addresses'))) {
                foreach ($request->get('customer_addresses') as $row) {
                    CustomerAddress::save([
                        'customer_id' => $customer->id,
                        'city' => $row['city'] ?? null,
                        'first_name' => $row['first_name'] ?? null,
                        'last_name' => $row['last_name'] ?? null,
                        'street' => $row['street'] ?? null,
                        'house_number' => $row['house_number'] ?? null,
                        'apt_number' => $row['apt_number'] ?? null,
                        'postal_code' => $row['postal_code'] ?? null,
                        'phone_one' => $row['phone_one'] ?? null,
                    ]);
                }
            } else {
                $customerAddress = new CustomerAddress();
                $customerAddress->customer_id = $customer->id;
                $customerAddress->phone_one = $request->get('phone') ?? null;
                $customerAddress->first_name = $request->get('first_name') ?? null;
                $customerAddress->last_name = $request->get('last_name') ?? null;

                $customerAddress->save();
            }
        });

        return Redirect::route('cms.customers.index');
    }

    /**
     * Display the specified resource.
     *
     * @param Customer $customer
     * @return Response
     */
    public function show(Customer $customer)
    {
        $view = View::make('erp::pages.customers.single');
        $formMethod = 'PUT';
        $formRoute = route('cms.customers.update', $customer->id);
        $customer->load('customer_addresses');
        $view->with(compact('formRoute', 'formMethod', 'customer'));
        return $view;
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Customer $customer
     * @return Response
     */
    public function edit(Customer $customer)
    {
        $view = View::make('erp::pages.customers.single');
        $formMethod = 'PUT';
        $formRoute = route('cms.customers.update', $customer->id);
        $customer->load('customer_addresses');
        $view->with(compact('formRoute', 'formMethod', 'customer'));
        return $view;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Customer $customer
     * @return Response
     * @throws Throwable
     */
    public function update(Request $request, Customer $customer)
    {
        $this->validate($request, [
            'email' => 'nullable|email',
            'phone' => 'required',
            'first_name' => 'required|string',
            'last_name' => 'required|string',
        ]);

        DB::transaction(function () use ($request, $customer) {
            $customer->email = $request->get('email') ?? null;
            $customer->phone = $request->get('phone') ?? null;
            $customer->last_name = $request->get('last_name') ?? null;
            $customer->first_name = $request->get('first_name') ?? null;
            $customer->customer_comment = $request->get('customer_comment') ?? null;
            $customer->update();

            if ($request->has('customer_addresses') && is_array($request->get('customer_addresses'))) {
                foreach ($request->get('customer_addresses') as $row) {
                    CustomerAddress::updateOrCreate([
                        'id' => $row['id'],
                        'customer_id' => $customer->id
                    ], [
                        'customer_id' => $customer->id,
                        'city' => $row['city'] ?? null,
                        'first_name' => $row['first_name'] ?? null,
                        'last_name' => $row['last_name'] ?? null,
                        'street' => $row['street'] ?? null,
                        'house_number' => $row['house_number'] ?? null,
                        'apt_number' => $row['apt_number'] ?? null,
                        'postal_code' => $row['postal_code'] ?? null,
                        'phone_one' => $row['phone_one'] ?? null,
                    ]);
                }
            }
            if (!CustomerAddress::where('customer_id', $customer->id)->exists()) {
                $customerAddress = new CustomerAddress();
                $customerAddress->customer_id = $customer->id;
                $customerAddress->phone_one = $request->get('phone') ?? null;
                $customerAddress->first_name = $request->get('first_name') ?? null;
                $customerAddress->last_name = $request->get('last_name') ?? null;

                $customerAddress->save();
            }
        });

        return Redirect::route('cms.customers.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Customer $customer
     * @return Response
     */
    public function destroy(Customer $customer)
    {
        //
    }
}
