<?php

namespace TDevAgency\Erp\Controllers;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\View;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Erp\Models\Delivery;
use TDevAgency\Erp\Models\DeliveryItem;
use TDevAgency\Erp\Models\ProviderTransaction;
use TDevAgency\Erp\Models\Provider;

class DeliveryController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function index()
    {
        $view = View::make('erp::pages.deliveries.index');
        $view->with('deliveries', Delivery::container()->search()->sortable(['created_at' => 'desc'])->paginate(30));
        return $view;
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $view = View::make('erp::pages.deliveries.single');
        $view->with('providers', Provider::container()->orderBy('name')->pluck('name', 'id'));
        $view->with('products', []);
        $view->with('delivery', new Delivery());
        $view->with('delivery_items', []);
        $view->with('delivery_transactions', []);
        $view->with('formRoute', route('cms.deliveries.store'));
        $view->with('formMethod', 'POST');

        return $view;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return RedirectResponse
     */
    public function store(Request $request)
    {
        $this->validateRequest();

        try {
            DB::transaction(function () use ($request) {
                $delivery = new Delivery();
                foreach ($delivery->getFillable() as $field) {
                    if ($request->has($field)) {
                        $delivery->setAttribute($field, $request->get($field));
                    } else {
                        $delivery->setAttribute($field, null);
                    }
                }
                $delivery->save();

                foreach ($request->get('delivery_items') as $item) {
                    $deliveryItem = new DeliveryItem();
                    foreach ($deliveryItem->getFillable() as $field) {
                        if (isset($item[$field])) {
                            $deliveryItem->setAttribute($field, $item[$field]);
                        } else {
                            $deliveryItem->setAttribute($field, null);
                        }
                    }
                    $deliveryItem->delivery_id = $delivery->id;
                    $deliveryItem->save();
                }
                if ($request->has('delivery_transactions')) {
                    foreach ($request->get('delivery_transactions') as $item) {
                        $providerTransaction = new ProviderTransaction();
                        foreach ($providerTransaction->getFillable() as $field) {
                            if (isset($item[$field])) {
                                $providerTransaction->setAttribute($field, $item[$field]);
                            } else {
                                $providerTransaction->setAttribute($field, null);
                            }
                        }
                        $providerTransaction->provider_id = $request->get('provider_id');
                        $providerTransaction->save();
                    }
                }
            });
        } catch (\Throwable $exception) {
            Log::error($exception);
            return Redirect::back()->withInput();
        }

        return Redirect::route('cms.deliveries.index')->with('toast', [
            'title' => trans('cms::admin.toast.created'),
            'status' => 'success'
        ]);
    }

    private function validateRequest()
    {
        $rules = [
            'provider_id' => 'required',
            'delivery_number' => 'nullable|string',
            'price_national_currency' => 'required|numeric',
            'price_in_currency' => 'nullable|numeric',
            'currency' => 'nullable|integer',
            'delivery_date' => 'date|required',
            'payment_deadline' => 'date|nullable',

            'delivery_items.*.product_id' => 'required',
            'delivery_items.*.price_per_unit' => 'required|numeric',
            'delivery_items.*.price_per_unit_in_currency' => 'nullable|numeric',
            'delivery_items.*.product_qty' => 'required|numeric',
        ];

        if ($this->request->has('delivery_transactions')) {
            $rules = array_merge($rules, ['delivery_transactions.*.payment_transaction_date' => 'required|date',
                'delivery_transactions.*.payment_amount' => 'required|numeric',
                'delivery_transactions.*.payment_amount_in_currency' => 'nullable|numeric']);
        }

        $this->validate($this->request, $rules, $this->validationCustomAttributes());
    }

    /**
     * @return array
     */
    private function validationCustomAttributes()
    {
        return [
            'provider_id' => trans('erp::erp.provider_id'),
            'currency' => trans('erp::erp.currency'),
            'delivery_number' => trans('erp::erp.delivery_number'),
            'price_in_currency' => trans('erp::erp.price_in_currency'),
        ];
    }

    /**
     * Display the specified resource.
     *
     * @param Delivery $delivery
     * @return RedirectResponse
     */
    public function show(Delivery $delivery)
    {
        return Redirect::route('cms.deliveries.edit', $delivery->id);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Delivery $delivery
     * @return \Illuminate\Contracts\View\View
     */
    public function edit(Delivery $delivery)
    {
        $delivery->load('delivery_items');
        $view = View::make('erp::pages.deliveries.single');
        $view->with('providers', Provider::container()->orderBy('name')->pluck('name', 'id'));
        $view->with('products', []);
        $view->with('delivery', $delivery);
        $view->with('delivery_items', $delivery->delivery_items);
        $view->with('formRoute', route('cms.deliveries.update', $delivery->id));
        $view->with('formMethod', 'PUT');

        return $view;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Delivery $delivery
     * @return RedirectResponse
     */
    public function update(Request $request, Delivery $delivery)
    {
        $this->validateRequest();

        try {
            DB::transaction(function () use ($request, $delivery) {
                foreach ($delivery->getFillable() as $field) {
                    if ($request->has($field)) {
                        $delivery->setAttribute($field, $request->get($field));
                    } else {
                        $delivery->setAttribute($field, null);
                    }
                }
                $delivery->update();

                $deliveryItem = new DeliveryItem();
                foreach ($request->get('delivery_items') as $item) {
                    $array = [];
                    foreach ($deliveryItem->getFillable() as $field) {
                        if (isset($item[$field])) {
                            $array[$field] = $item[$field];
                        } else {
                            $array[$field] = null;
                        }
                    }
                    $array['delivery_id'] = $delivery->id;

                    DeliveryItem::updateOrCreate(['id' => $item['id'], 'delivery_id' => $delivery->id], $array);
                }
            });
        } catch (\Throwable $exception) {
            Log::error($exception);
            return Redirect::back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_created'),
                'status' => 'danger'
            ]);
        }

        return Redirect::route('cms.deliveries.index')->with('toast', [
            'title' => trans('cms::admin.toast.created'),
            'status' => 'success'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Delivery $delivery
     * @return Response
     */
    public function destroy(Delivery $delivery)
    {
        //
    }
}
