<?php

namespace TDevAgency\Erp\Controllers;

use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use Illuminate\Validation\Rule;
use TDevAgency\Cms\Controllers\Controller;
use Illuminate\Http\Request;
use TDevAgency\Erp\Events\ProductChangeEvent;
use TDevAgency\Erp\Models\Product;
use TDevAgency\Erp\Models\Manufacture;

class ProductsController extends Controller
{
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function create(): \Illuminate\Contracts\View\View
    {
        $view    = View::make('erp::products.single');
        $product = new Product();
        $view->with('product', $product);
        $view->with('formRoute', route('cms.products.store'));
        $view->with('formMethod', 'POST');
        $view->with('manufactures', Manufacture::pluck('manufacture_name', 'id'));
        return $view;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy($id)
    {
        $product = Product::container()->findOrFail($id);

        $path = sprintf('%s/%s', $product->getTable(), $product->getId());
        if (Storage::disk('public')->exists($path)) {
            Storage::disk('public')->deleteDirectory($path);
        }

        $product->delete();

        return Redirect::back()->with('toast', [
            'title'  => trans('cms::admin.toast.deleted'),
            'status' => 'success'
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param $id
     * @return \Illuminate\Contracts\View\View
     */
    public function edit($id)
    {
        $product = Product::container()->findOrFail($id);
        $view    = View::make('erp::products.single');
        //        $product = Product::findOrFail($id);
        $view->with('product', $product);
        $view->with('formRoute', route('cms.products.update', $product->id));
        $view->with('formMethod', 'PUT');
        $view->with('manufactures', Manufacture::pluck('manufacture_name', 'id'));
        return $view;
    }

    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\View\View
     */
    public function index(Request $request)
    {
        $view  = View::make('erp::products.index');
        $items = Product::container()->qty()->filter()->search()->sortable('sku')->with('manufacture')->paginate(30);
        $view->with('manufactures', Manufacture::pluck('manufacture_name', 'id'));
        $view->with('items', $items);
        return $view;
    }

    /**
     * Display the specified resource.
     *
     * @param Product $product
     * @return Response
     */
    public function show(Product $product)
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return RedirectResponse
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'sku'              => ['required',
                                   'string',
                                   Rule::unique('products')->where(function ($query) use ($request) {
                                       return $query
                                           ->where('sku', $request->get('sku'))
                                           ->where('container_id', Auth::guard('admin_user')->user()->container->id);
                                   })],
            'product_name'     => 'required|string',
            'price'            => 'numeric|min:0',
            'special_price'    => 'numeric|min:0',
            'product_weight'   => 'nullable|numeric|min:0',
            'guarantee_period' => 'nullable|numeric|min:0',
            'product_width'    => 'nullable|numeric|min:0',
            'product_height'   => 'nullable|numeric|min:0',
            'product_length'   => 'nullable|numeric|min:0',
            'manufacture_id'   => 'required|integer',
            'image'            => 'nullable|image',
        ]);

        $product = new Product();

        foreach ($product->getFillable() as $field) {
            if ($request->has($field)) {
                $product->setAttribute($field, $request->get($field));
            } else {
                $product->setAttribute($field, null);
            }
        }

        $product->save();
        if ($request->hasFile('image')) {
            $product->image = App::make('Erp\ImageHelper')->generateImages($request->file('image'), $product);
            $product->update();
        }

        Event::dispatch(new ProductChangeEvent($product));
        return Redirect::route('cms.products.index')->with('toast', [
            'title'  => trans('cms::admin.toast.created'),
            'status' => 'success'
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param $id
     * @return RedirectResponse
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'sku'              => ['required',
                                   'string',
                                   Rule::unique('products')->where(function ($query) use ($request) {
                                       return $query
                                           ->where('sku', $request->get('sku'))
                                           ->where('container_id', Auth::guard('admin_user')->user()->container->id);
                                   })->ignore($id)],
            'product_name'     => 'required|string',
            'price'            => 'numeric|min:0',
            'special_price'    => 'numeric|min:0',
            'product_weight'   => 'nullable|numeric|min:0',
            'product_width'    => 'nullable|numeric|min:0',
            'guarantee_period' => 'nullable|numeric|min:0',
            'product_height'   => 'nullable|numeric|min:0',
            'product_length'   => 'nullable|numeric|min:0',
            'manufacture_id'   => 'required|integer',
            'image'            => 'nullable|image',
        ]);

        $product = Product::container()->findOrFail($id);

        foreach ($product->getFillable() as $field) {
            if ($request->has($field)) {
                $product->setAttribute($field, $request->get($field));
            } else {
                $product->setAttribute($field, null);
            }
        }

        $product->update();
        if ($request->hasFile('image')) {
            $product->image = App::make('Erp\ImageHelper')->generateImages($request->file('image'), $product);
            $product->update();
        }
        Event::dispatch(new ProductChangeEvent($product));
        return Redirect::route('cms.products.index')->with('toast', [
            'title'  => trans('cms::admin.toast.updated'),
            'status' => 'success'
        ]);
    }
}
